@@ -42,6 +42,22 @@ struct _MMBroadbandBearerNovatelLtePrivate {
     guint connection_poller;
 };
 
+static gchar *
+normalize_qmistatus (const gchar *status)
+{
+    gchar *normalized_status, *iter;
+
+    if (!status)
+        return NULL;
+
+    normalized_status = g_strdup (status);
+    for (iter = normalized_status; *iter; iter++)
+        if (g_ascii_isspace (*iter))
+            *iter = ' ';
+
+    return normalized_status;
+}
+
 /*****************************************************************************/
 /* 3GPP Connection sequence */
 
@@ -55,69 +71,29 @@ typedef struct {
     gint retries;
 } DetailedConnectContext;
 
-static DetailedConnectContext *
-detailed_connect_context_new (MMBroadbandBearer *self,
-                              MMBroadbandModem *modem,
-                              MMAtSerialPort *primary,
-                              MMPort *data,
-                              GCancellable *cancellable,
-                              GAsyncReadyCallback callback,
-                              gpointer user_data)
-{
-    DetailedConnectContext *ctx;
-
-    ctx = g_new0 (DetailedConnectContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->modem = MM_BASE_MODEM (g_object_ref (modem));
-    ctx->primary = g_object_ref (primary);
-    ctx->data = g_object_ref (data);
-    /* NOTE:
-     * We don't currently support cancelling AT commands, so we'll just check
-     * whether the operation is to be cancelled at each step. */
-    ctx->cancellable = g_object_ref (cancellable);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             detailed_connect_context_new);
-    ctx->retries = 4;
-    return ctx;
-}
-
 static void
 detailed_connect_context_complete_and_free (DetailedConnectContext *ctx)
 {
     g_simple_async_result_complete_in_idle (ctx->result);
     g_object_unref (ctx->result);
     g_object_unref (ctx->cancellable);
-    g_object_unref (ctx->data);
+    if (ctx->data)
+        g_object_unref (ctx->data);
     g_object_unref (ctx->primary);
     g_object_unref (ctx->modem);
     g_object_unref (ctx->self);
-    g_free (ctx);
+    g_slice_free (DetailedConnectContext, ctx);
 }
 
-static gboolean
+static MMBearerConnectResult *
 connect_3gpp_finish (MMBroadbandBearer *self,
                      GAsyncResult *res,
-                     MMPort **data,
-                     MMBearerIpConfig **ipv4_config,
-                     MMBearerIpConfig **ipv6_config,
                      GError **error)
 {
-    MMBearerIpConfig *config;
-
     if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
-        return FALSE;
+        return NULL;
 
-    config = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res));
-
-    /* In the default implementation, we assume only IPv4 is supported */
-    *ipv4_config = g_object_ref (config);
-    *ipv6_config = NULL;
-    /* We used the input suggested data port */
-    *data = NULL;
-
-    return TRUE;
+    return mm_bearer_connect_result_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
 }
 
 static gboolean connect_3gpp_qmistatus (DetailedConnectContext *ctx);
@@ -186,6 +162,7 @@ connect_3gpp_qmistatus_ready (MMBaseModem *modem,
                               DetailedConnectContext *ctx)
 {
     const gchar *result;
+    gchar *normalized_result;
     GError *error = NULL;
 
     result = mm_base_modem_at_command_finish (MM_BASE_MODEM (ctx->modem),
@@ -209,9 +186,11 @@ connect_3gpp_qmistatus_ready (MMBaseModem *modem,
                                                                     ctx->self);
         config = mm_bearer_ip_config_new ();
         mm_bearer_ip_config_set_method (config, MM_BEARER_IP_METHOD_DHCP);
-        g_simple_async_result_set_op_res_gpointer (ctx->result,
-                                                   config,
-                                                   (GDestroyNotify)g_object_unref);
+        g_simple_async_result_set_op_res_gpointer (
+            ctx->result,
+            mm_bearer_connect_result_new (ctx->data, config, NULL),
+            (GDestroyNotify)mm_bearer_connect_result_unref);
+        g_object_unref (config);
         detailed_connect_context_complete_and_free (ctx);
         return;
     }
@@ -226,10 +205,13 @@ connect_3gpp_qmistatus_ready (MMBaseModem *modem,
     }
 
     /* Already exhausted all retries */
+    normalized_result = normalize_qmistatus (result);
     g_simple_async_result_set_error (ctx->result,
                                      MM_CORE_ERROR,
                                      MM_CORE_ERROR_FAILED,
-                                     "%s", result);
+                                     "QMI connect failed: %s",
+                                     normalized_result);
+    g_free (normalized_result);
     detailed_connect_context_complete_and_free (ctx);
 }
 
@@ -275,28 +257,12 @@ connect_3gpp_qmiconnect_ready (MMBaseModem *modem,
 }
 
 static void
-connect_3gpp (MMBroadbandBearer *bearer,
-              MMBroadbandModem *modem,
-              MMAtSerialPort *primary,
-              MMAtSerialPort *secondary,
-              MMPort *data,
-              GCancellable *cancellable,
-              GAsyncReadyCallback callback,
-              gpointer user_data)
+connect_3gpp_authenticate (DetailedConnectContext *ctx)
 {
-    DetailedConnectContext *ctx;
-    gchar *command, *apn, *user, *password;
     MMBearerProperties *config;
+    gchar *command, *apn, *user, *password;
 
-    ctx = detailed_connect_context_new (bearer,
-                                        modem,
-                                        primary,
-                                        data,
-                                        cancellable,
-                                        callback,
-                                        user_data);
-
-    config = mm_bearer_peek_config (MM_BEARER (bearer));
+    config = mm_bearer_peek_config (MM_BEARER (ctx->self));
     apn = mm_at_serial_port_quote_string (mm_bearer_properties_get_apn (config));
     user = mm_at_serial_port_quote_string (mm_bearer_properties_get_user (config));
     password = mm_at_serial_port_quote_string (mm_bearer_properties_get_password (config));
@@ -315,6 +281,44 @@ connect_3gpp (MMBroadbandBearer *bearer,
     g_free (command);
 }
 
+static void
+connect_3gpp (MMBroadbandBearer *self,
+              MMBroadbandModem *modem,
+              MMAtSerialPort *primary,
+              MMAtSerialPort *secondary,
+              GCancellable *cancellable,
+              GAsyncReadyCallback callback,
+              gpointer user_data)
+{
+    DetailedConnectContext *ctx;
+
+    ctx = g_slice_new0 (DetailedConnectContext);
+    ctx->self = g_object_ref (self);
+    ctx->modem = MM_BASE_MODEM (g_object_ref (modem));
+    ctx->primary = g_object_ref (primary);
+    ctx->cancellable = g_object_ref (cancellable);
+    ctx->result = g_simple_async_result_new (G_OBJECT (self),
+                                             callback,
+                                             user_data,
+                                             connect_3gpp);
+    ctx->retries = 60;
+
+    /* Get a 'net' data port */
+    ctx->data = mm_base_modem_get_best_data_port (MM_BASE_MODEM (modem),
+                                                  MM_PORT_TYPE_NET);
+    if (!ctx->data) {
+        g_simple_async_result_set_error (
+            ctx->result,
+            MM_CORE_ERROR,
+            MM_CORE_ERROR_CONNECTED,
+            "Couldn't connect: no available net port available");
+        detailed_connect_context_complete_and_free (ctx);
+        return;
+    }
+
+    connect_3gpp_authenticate (ctx);
+}
+
 /*****************************************************************************/
 /* 3GPP Disonnection sequence */
 
@@ -325,6 +329,7 @@ typedef struct {
     MMAtSerialPort *secondary;
     MMPort *data;
     GSimpleAsyncResult *result;
+    gint retries;
 } DetailedDisconnectContext;
 
 static DetailedDisconnectContext *
@@ -348,6 +353,7 @@ detailed_disconnect_context_new (MMBroadbandBearer *self,
                                              callback,
                                              user_data,
                                              detailed_disconnect_context_new);
+    ctx->retries = 60;
     return ctx;
 }
 
@@ -373,34 +379,77 @@ disconnect_3gpp_finish (MMBroadbandBearer *self,
     return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
 }
 
+static gboolean disconnect_3gpp_qmistatus (DetailedDisconnectContext *ctx);
+
 static void
-disconnect_3gpp_status_complete (MMBaseModem *modem,
-                                 GAsyncResult *res,
-                                 DetailedDisconnectContext *ctx)
+disconnect_3gpp_status_ready (MMBaseModem *modem,
+                              GAsyncResult *res,
+                              DetailedDisconnectContext *ctx)
 {
     const gchar *result;
     GError *error = NULL;
+    gboolean is_connected = FALSE;
 
     result = mm_base_modem_at_command_finish (MM_BASE_MODEM (modem),
                                               res,
                                               &error);
-    if (error) {
-        mm_dbg("QMI connection status failed: %s", error->message);
+    if (result) {
+        mm_dbg ("QMI connection status: %s", result);
+        if (is_qmistatus_disconnected (result)) {
+            g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
+            detailed_disconnect_context_complete_and_free (ctx);
+            return;
+        } else if (is_qmistatus_connected (result)) {
+            is_connected = TRUE;
+        }
+    } else {
+        mm_dbg ("QMI connection status failed: %s", error->message);
         g_error_free (error);
+        result = "Unknown error";
     }
 
-    if (result && is_qmistatus_disconnected (result))
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-    else
+    if (ctx->retries > 0) {
+        ctx->retries--;
+        mm_dbg ("Retrying status check in a second. %d retries left.",
+                ctx->retries);
+        g_timeout_add_seconds (1, (GSourceFunc)disconnect_3gpp_qmistatus, ctx);
+        return;
+    }
+
+    /* If $NWQMISTATUS reports a CONNECTED QMI state, returns an error such that
+     * the modem state remains 'connected'. Otherwise, assumes the modem is
+     * disconnected from the network successfully. */
+    if (is_connected) {
+        gchar *normalized_result;
+
+        normalized_result = normalize_qmistatus (result);
         g_simple_async_result_set_error (ctx->result,
                                          MM_CORE_ERROR,
                                          MM_CORE_ERROR_FAILED,
-                                         "Error checking if disconnected (%s)",
-                                         result ? result : "no result");
+                                         "QMI disconnect failed: %s",
+                                         normalized_result);
+        g_free (normalized_result);
+    } else
+        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
 
     detailed_disconnect_context_complete_and_free (ctx);
 }
 
+static gboolean
+disconnect_3gpp_qmistatus (DetailedDisconnectContext *ctx)
+{
+    mm_base_modem_at_command (
+        ctx->modem,
+        "$NWQMISTATUS",
+        3, /* timeout */
+        FALSE, /* allow_cached */
+        (GAsyncReadyCallback)disconnect_3gpp_status_ready, /* callback */
+        ctx); /* user_data */
+
+    return FALSE;
+}
+
+
 static void
 disconnect_3gpp_check_status (MMBaseModem *modem,
                               GAsyncResult *res,
@@ -416,13 +465,7 @@ disconnect_3gpp_check_status (MMBaseModem *modem,
         g_error_free (error);
     }
 
-    mm_base_modem_at_command (
-        ctx->modem,
-        "$NWQMISTATUS",
-        3, /* timeout */
-        FALSE, /* allow_cached */
-        (GAsyncReadyCallback)disconnect_3gpp_status_complete,
-        ctx); /* user_data */
+    disconnect_3gpp_qmistatus (ctx);
 }
 
 static void
