@@ -37,11 +37,12 @@
 static void iface_modem_init (MMIfaceModem *iface);
 static void iface_modem_3gpp_init (MMIfaceModem3gpp *iface);
 
+static MMIfaceModem *iface_modem_parent;
 static MMIfaceModem3gpp *iface_modem_3gpp_parent;
 
 G_DEFINE_TYPE_EXTENDED (MMBroadbandModemSimtech, mm_broadband_modem_simtech, MM_TYPE_BROADBAND_MODEM, 0,
                         G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM, iface_modem_init)
-                        G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_3GPP, iface_modem_3gpp_init));
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_3GPP, iface_modem_3gpp_init))
 
 /*****************************************************************************/
 /* Setup/Cleanup unsolicited events (3GPP interface) */
@@ -444,23 +445,104 @@ load_access_technologies (MMIfaceModem *self,
         result);
 }
 
+/*****************************************************************************/
+/* Load supported modes (Modem interface) */
+
+static GArray *
+load_supported_modes_finish (MMIfaceModem *self,
+                             GAsyncResult *res,
+                             GError **error)
+{
+    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
+        return NULL;
+
+    return g_array_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
+}
+
+static void
+parent_load_supported_modes_ready (MMIfaceModem *self,
+                                   GAsyncResult *res,
+                                   GSimpleAsyncResult *simple)
+{
+    GError *error = NULL;
+    GArray *all;
+    GArray *combinations;
+    GArray *filtered;
+    MMModemModeCombination mode;
+
+    all = iface_modem_parent->load_supported_modes_finish (self, res, &error);
+    if (!all) {
+        g_simple_async_result_take_error (simple, error);
+        g_simple_async_result_complete (simple);
+        g_object_unref (simple);
+        return;
+    }
+
+    /* Build list of combinations */
+    combinations = g_array_sized_new (FALSE, FALSE, sizeof (MMModemModeCombination), 5);
+    /* 2G only */
+    mode.allowed = MM_MODEM_MODE_2G;
+    mode.preferred = MM_MODEM_MODE_NONE;
+    g_array_append_val (combinations, mode);
+    /* 3G only */
+    mode.allowed = MM_MODEM_MODE_3G;
+    mode.preferred = MM_MODEM_MODE_NONE;
+    g_array_append_val (combinations, mode);
+    /* 2G and 3G */
+    mode.allowed = (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G);
+    mode.preferred = MM_MODEM_MODE_NONE;
+    g_array_append_val (combinations, mode);
+    /* 2G and 3G, 2G preferred */
+    mode.allowed = (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G);
+    mode.preferred = MM_MODEM_MODE_2G;
+    g_array_append_val (combinations, mode);
+    /* 2G and 3G, 3G preferred */
+    mode.allowed = (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G);
+    mode.preferred = MM_MODEM_MODE_3G;
+    g_array_append_val (combinations, mode);
+
+    /* Filter out those unsupported modes */
+    filtered = mm_filter_supported_modes (all, combinations);
+    g_array_unref (all);
+    g_array_unref (combinations);
+
+    g_simple_async_result_set_op_res_gpointer (simple, filtered, (GDestroyNotify) g_array_unref);
+    g_simple_async_result_complete (simple);
+    g_object_unref (simple);
+}
+
+static void
+load_supported_modes (MMIfaceModem *self,
+                      GAsyncReadyCallback callback,
+                      gpointer user_data)
+{
+    /* Run parent's loading */
+    iface_modem_parent->load_supported_modes (
+        MM_IFACE_MODEM (self),
+        (GAsyncReadyCallback)parent_load_supported_modes_ready,
+        g_simple_async_result_new (G_OBJECT (self),
+                                   callback,
+                                   user_data,
+                                   load_supported_modes));
+}
+
 /*****************************************************************************/
 /* Load initial allowed/preferred modes (Modem interface) */
 
 typedef struct {
     MMModemMode allowed;
     MMModemMode preferred;
-} LoadAllowedModesResult;
+} LoadCurrentModesResult;
 
 typedef struct {
     MMBroadbandModemSimtech *self;
     GSimpleAsyncResult *result;
     gint acqord;
     gint modepref;
-} LoadAllowedModesContext;
+} LoadCurrentModesContext;
 
 static void
-load_allowed_modes_context_complete_and_free (LoadAllowedModesContext *ctx)
+load_current_modes_context_complete_and_free (LoadCurrentModesContext *ctx)
 {
     g_simple_async_result_complete (ctx->result);
     g_object_unref (ctx->result);
@@ -469,13 +551,13 @@ load_allowed_modes_context_complete_and_free (LoadAllowedModesContext *ctx)
 }
 
 static gboolean
-load_allowed_modes_finish (MMIfaceModem *self,
+load_current_modes_finish (MMIfaceModem *self,
                            GAsyncResult *res,
                            MMModemMode *allowed,
                            MMModemMode *preferred,
                            GError **error)
 {
-    LoadAllowedModesResult *result;
+    LoadCurrentModesResult *result;
 
     if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
         return FALSE;
@@ -489,9 +571,9 @@ load_allowed_modes_finish (MMIfaceModem *self,
 static void
 cnmp_query_ready (MMBroadbandModemSimtech *self,
                   GAsyncResult *res,
-                  LoadAllowedModesContext *ctx)
+                  LoadCurrentModesContext *ctx)
 {
-    LoadAllowedModesResult *result;
+    LoadCurrentModesResult *result;
     const gchar *response, *p;
     GError *error = NULL;
 
@@ -499,7 +581,7 @@ cnmp_query_ready (MMBroadbandModemSimtech *self,
     if (!response) {
         /* Let the error be critical. */
         g_simple_async_result_take_error (ctx->result, error);
-        load_allowed_modes_context_complete_and_free (ctx);
+        load_current_modes_context_complete_and_free (ctx);
         return;
     }
 
@@ -511,11 +593,11 @@ cnmp_query_ready (MMBroadbandModemSimtech *self,
             MM_CORE_ERROR_FAILED,
             "Failed to parse the mode preference response: '%s'",
             response);
-        load_allowed_modes_context_complete_and_free (ctx);
+        load_current_modes_context_complete_and_free (ctx);
         return;
     }
 
-    result = g_new (LoadAllowedModesResult, 1);
+    result = g_new (LoadCurrentModesResult, 1);
     result->allowed = MM_MODEM_MODE_NONE;
     result->preferred = MM_MODEM_MODE_NONE;
 
@@ -543,7 +625,7 @@ cnmp_query_ready (MMBroadbandModemSimtech *self,
                 MM_CORE_ERROR_FAILED,
                 "Unknown acquisition order preference: '%d'",
                 ctx->acqord);
-            load_allowed_modes_context_complete_and_free (ctx);
+            load_current_modes_context_complete_and_free (ctx);
             return;
         }
         break;
@@ -567,7 +649,7 @@ cnmp_query_ready (MMBroadbandModemSimtech *self,
             MM_CORE_ERROR_FAILED,
             "Unknown mode preference: '%d'",
             ctx->modepref);
-        load_allowed_modes_context_complete_and_free (ctx);
+        load_current_modes_context_complete_and_free (ctx);
         return;
     }
 
@@ -575,13 +657,13 @@ cnmp_query_ready (MMBroadbandModemSimtech *self,
     g_simple_async_result_set_op_res_gpointer (ctx->result,
                                                result,
                                                g_free);
-    load_allowed_modes_context_complete_and_free (ctx);
+    load_current_modes_context_complete_and_free (ctx);
 }
 
 static void
 cnaop_query_ready (MMBroadbandModemSimtech *self,
                    GAsyncResult *res,
-                   LoadAllowedModesContext *ctx)
+                   LoadCurrentModesContext *ctx)
 {
     const gchar *response, *p;
     GError *error = NULL;
@@ -590,7 +672,7 @@ cnaop_query_ready (MMBroadbandModemSimtech *self,
     if (!response) {
         /* Let the error be critical. */
         g_simple_async_result_take_error (ctx->result, error);
-        load_allowed_modes_context_complete_and_free (ctx);
+        load_current_modes_context_complete_and_free (ctx);
         return;
     }
 
@@ -605,7 +687,7 @@ cnaop_query_ready (MMBroadbandModemSimtech *self,
             MM_CORE_ERROR_FAILED,
             "Failed to parse the acquisition order response: '%s'",
             response);
-        load_allowed_modes_context_complete_and_free (ctx);
+        load_current_modes_context_complete_and_free (ctx);
         return;
     }
 
@@ -619,18 +701,18 @@ cnaop_query_ready (MMBroadbandModemSimtech *self,
 }
 
 static void
-load_allowed_modes (MMIfaceModem *self,
+load_current_modes (MMIfaceModem *self,
                     GAsyncReadyCallback callback,
                     gpointer user_data)
 {
-    LoadAllowedModesContext *ctx;
+    LoadCurrentModesContext *ctx;
 
-    ctx = g_new (LoadAllowedModesContext, 1);
+    ctx = g_new (LoadCurrentModesContext, 1);
     ctx->self = g_object_ref (self);
     ctx->result = g_simple_async_result_new (G_OBJECT (self),
                                              callback,
                                              user_data,
-                                             load_allowed_modes);
+                                             load_current_modes);
     ctx->acqord = -1;
     ctx->modepref = -1;
 
@@ -651,10 +733,10 @@ typedef struct {
     GSimpleAsyncResult *result;
     guint nmp;   /* mode preference */
     guint naop;  /* acquisition order */
-} SetAllowedModesContext;
+} SetCurrentModesContext;
 
 static void
-set_allowed_modes_context_complete_and_free (SetAllowedModesContext *ctx)
+set_current_modes_context_complete_and_free (SetCurrentModesContext *ctx)
 {
     g_simple_async_result_complete_in_idle (ctx->result);
     g_object_unref (ctx->result);
@@ -663,7 +745,7 @@ set_allowed_modes_context_complete_and_free (SetAllowedModesContext *ctx)
 }
 
 static gboolean
-set_allowed_modes_finish (MMIfaceModem *self,
+set_current_modes_finish (MMIfaceModem *self,
                           GAsyncResult *res,
                           GError **error)
 {
@@ -673,7 +755,7 @@ set_allowed_modes_finish (MMIfaceModem *self,
 static void
 cnaop_set_ready (MMBaseModem *self,
                  GAsyncResult *res,
-                 SetAllowedModesContext *ctx)
+                 SetCurrentModesContext *ctx)
 {
     GError *error = NULL;
 
@@ -684,13 +766,13 @@ cnaop_set_ready (MMBaseModem *self,
     else
         g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
 
-    set_allowed_modes_context_complete_and_free (ctx);
+    set_current_modes_context_complete_and_free (ctx);
 }
 
 static void
 cnmp_set_ready (MMBaseModem *self,
                 GAsyncResult *res,
-                SetAllowedModesContext *ctx)
+                SetCurrentModesContext *ctx)
 {
     GError *error = NULL;
     gchar *command;
@@ -699,7 +781,7 @@ cnmp_set_ready (MMBaseModem *self,
     if (error) {
         /* Let the error be critical. */
         g_simple_async_result_take_error (ctx->result, error);
-        set_allowed_modes_context_complete_and_free (ctx);
+        set_current_modes_context_complete_and_free (ctx);
         return;
     }
 
@@ -715,35 +797,44 @@ cnmp_set_ready (MMBaseModem *self,
 }
 
 static void
-set_allowed_modes (MMIfaceModem *self,
+set_current_modes (MMIfaceModem *self,
                    MMModemMode allowed,
                    MMModemMode preferred,
                    GAsyncReadyCallback callback,
                    gpointer user_data)
 {
-    SetAllowedModesContext *ctx;
+    SetCurrentModesContext *ctx;
     gchar *command;
 
-    ctx = g_new (SetAllowedModesContext, 1);
+    ctx = g_new (SetCurrentModesContext, 1);
     ctx->self = g_object_ref (self);
     ctx->result = g_simple_async_result_new (G_OBJECT (self),
                                              callback,
                                              user_data,
-                                             set_allowed_modes);
+                                             set_current_modes);
 
-    ctx->nmp = 2;  /* automatic mode preference */
-    ctx->naop = 0; /* automatic acquisition order */
+    /* Defaults: automatic search */
+    ctx->nmp = 2;
+    ctx->naop = 0;
 
-    if (allowed == MM_MODEM_MODE_2G)
+    if (allowed == MM_MODEM_MODE_ANY &&
+        preferred == MM_MODEM_MODE_NONE) {
+        /* defaults nmp and naop */
+    } else if (allowed == MM_MODEM_MODE_2G) {
         ctx->nmp = 13;
-    else if (allowed == MM_MODEM_MODE_3G)
+        ctx->naop = 0;
+    } else if (allowed == MM_MODEM_MODE_3G) {
         ctx->nmp = 14;
-    else if (allowed == (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G)) {
+        ctx->naop = 0;
+    } else if (allowed == (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G)) {
+        /* default nmp */
         if (preferred == MM_MODEM_MODE_2G)
             ctx->naop = 3;
         else if (preferred == MM_MODEM_MODE_3G)
             ctx->naop = 2;
-        /* else, auto */
+        else
+            /* default naop */
+            ctx->naop = 0;
     } else {
         gchar *allowed_str;
         gchar *preferred_str;
@@ -760,7 +851,7 @@ set_allowed_modes (MMIfaceModem *self,
         g_free (allowed_str);
         g_free (preferred_str);
 
-        set_allowed_modes_context_complete_and_free (ctx);
+        set_current_modes_context_complete_and_free (ctx);
         return;
     }
 
@@ -830,12 +921,16 @@ iface_modem_3gpp_init (MMIfaceModem3gpp *iface)
 static void
 iface_modem_init (MMIfaceModem *iface)
 {
+    iface_modem_parent = g_type_interface_peek_parent (iface);
+
     iface->load_access_technologies = load_access_technologies;
     iface->load_access_technologies_finish = load_access_technologies_finish;
-    iface->load_allowed_modes = load_allowed_modes;
-    iface->load_allowed_modes_finish = load_allowed_modes_finish;
-    iface->set_allowed_modes = set_allowed_modes;
-    iface->set_allowed_modes_finish = set_allowed_modes_finish;
+    iface->load_supported_modes = load_supported_modes;
+    iface->load_supported_modes_finish = load_supported_modes_finish;
+    iface->load_current_modes = load_current_modes;
+    iface->load_current_modes_finish = load_current_modes_finish;
+    iface->set_current_modes = set_current_modes;
+    iface->set_current_modes_finish = set_current_modes_finish;
 }
 
 static void
