@@ -115,6 +115,14 @@ handle_activate_auth_ready (MMBaseModem *self,
         return;
     }
 
+    /* If we're already activated, nothing to do */
+    if (mm_gdbus_modem_cdma_get_activation_state (ctx->skeleton) == MM_MODEM_CDMA_ACTIVATION_STATE_ACTIVATED) {
+        mm_dbg ("Modem is already activated");
+        mm_gdbus_modem_cdma_complete_activate (ctx->skeleton, ctx->invocation);
+        handle_activate_context_free (ctx);
+        return;
+    }
+
     /* If activating OTA is not implemented, report an error */
     if (!MM_IFACE_MODEM_CDMA_GET_INTERFACE (self)->activate ||
         !MM_IFACE_MODEM_CDMA_GET_INTERFACE (self)->activate_finish) {
@@ -242,7 +250,7 @@ handle_activate_manual_ready (MMIfaceModemCdma *self,
     if (!MM_IFACE_MODEM_CDMA_GET_INTERFACE (self)->activate_manual_finish (self, res,&error))
         g_dbus_method_invocation_take_error (ctx->invocation, error);
     else
-        mm_gdbus_modem_cdma_complete_activate (ctx->skeleton, ctx->invocation);
+        mm_gdbus_modem_cdma_complete_activate_manual (ctx->skeleton, ctx->invocation);
 
     handle_activate_manual_context_free (ctx);
 }
@@ -261,9 +269,17 @@ handle_activate_manual_auth_ready (MMBaseModem *self,
         return;
     }
 
+    /* If we're already activated, nothing to do */
+    if (mm_gdbus_modem_cdma_get_activation_state (ctx->skeleton) == MM_MODEM_CDMA_ACTIVATION_STATE_ACTIVATED) {
+        mm_dbg ("Modem is already activated");
+        mm_gdbus_modem_cdma_complete_activate_manual (ctx->skeleton, ctx->invocation);
+        handle_activate_manual_context_free (ctx);
+        return;
+    }
+
     /* If activating OTA is not implemented, report an error */
-    if (!MM_IFACE_MODEM_CDMA_GET_INTERFACE (self)->activate ||
-        !MM_IFACE_MODEM_CDMA_GET_INTERFACE (self)->activate_finish) {
+    if (!MM_IFACE_MODEM_CDMA_GET_INTERFACE (self)->activate_manual ||
+        !MM_IFACE_MODEM_CDMA_GET_INTERFACE (self)->activate_manual_finish) {
         g_dbus_method_invocation_return_error (ctx->invocation,
                                                MM_CORE_ERROR,
                                                MM_CORE_ERROR_UNSUPPORTED,
@@ -418,6 +434,7 @@ typedef enum {
 
     REGISTRATION_CHECK_STEP_QCDM_CALL_MANAGER_STATE,
     REGISTRATION_CHECK_STEP_QCDM_HDR_STATE,
+    REGISTRATION_CHECK_STEP_QCDM_CDMA1X_SERVING_SYSTEM,
     REGISTRATION_CHECK_STEP_QCDM_LAST,
 
     REGISTRATION_CHECK_STEP_AT_CDMA_SERVICE_STATUS,
@@ -543,9 +560,9 @@ get_hdr_state_ready (MMIfaceModemCdma *self,
     if (!MM_IFACE_MODEM_CDMA_GET_INTERFACE (self)->get_hdr_state_finish (
             self,
             res,
+            &ctx->hdr_hybrid_mode,
             &ctx->hdr_session_state,
             &ctx->hdr_almp_state,
-            &ctx->hdr_hybrid_mode,
             &error)) {
         mm_dbg ("Could not get HDR state: %s", error->message);
         g_error_free (error);
@@ -563,6 +580,11 @@ get_hdr_state_ready (MMIfaceModemCdma *self,
 static void
 parse_qcdm_results (RunRegistrationChecksContext *ctx)
 {
+    mm_dbg ("QCDM CM System Mode: %d", ctx->call_manager_system_mode);
+    mm_dbg ("QCDM HDR Hybrid Mode: %d", ctx->hdr_hybrid_mode);
+    mm_dbg ("QCDM HDR Session State: %d", ctx->hdr_session_state);
+    mm_dbg ("QCDM HDR ALMP State: %d", ctx->hdr_almp_state);
+
     /* Set QCDM-obtained registration info */
     switch (ctx->call_manager_system_mode) {
     case QCDM_CMD_CM_SUBSYS_STATE_INFO_SYSTEM_MODE_CDMA:
@@ -630,6 +652,8 @@ get_cdma1x_serving_system_ready (MMIfaceModemCdma *self,
 {
     GError *error = NULL;
 
+    /* Note: used for *both* AT and QCDM serving system checks */
+
     if (!MM_IFACE_MODEM_CDMA_GET_INTERFACE (self)->get_cdma1x_serving_system_finish (
             self,
             res,
@@ -764,6 +788,22 @@ registration_check_step (RunRegistrationChecksContext *ctx)
         /* Fall down to next step */
         ctx->step++;
 
+    case REGISTRATION_CHECK_STEP_QCDM_CDMA1X_SERVING_SYSTEM:
+        /* We only care about SID/NID here; nothing to do with registration
+         * state.
+         */
+        if (MM_IFACE_MODEM_CDMA_GET_INTERFACE (ctx->self)->get_cdma1x_serving_system &&
+            MM_IFACE_MODEM_CDMA_GET_INTERFACE (ctx->self)->get_cdma1x_serving_system_finish) {
+            MM_IFACE_MODEM_CDMA_GET_INTERFACE (ctx->self)->get_cdma1x_serving_system (
+                ctx->self,
+                (GAsyncReadyCallback)get_cdma1x_serving_system_ready,
+                ctx);
+            return;
+        }
+        mm_dbg ("  Skipping CDMA1x Serving System check");
+        /* Fall down to next step */
+        ctx->step++;
+
     case REGISTRATION_CHECK_STEP_QCDM_LAST:
         /* When we get all QCDM results, parse them */
         parse_qcdm_results (ctx);
@@ -824,8 +864,8 @@ registration_check_step (RunRegistrationChecksContext *ctx)
             MM_IFACE_MODEM_CDMA_GET_INTERFACE (ctx->self)->get_detailed_registration_state &&
             MM_IFACE_MODEM_CDMA_GET_INTERFACE (ctx->self)->get_detailed_registration_state_finish) {
             /* We pass the CDMA1x/EVDO registration states we got up to now.
-             * If the implementation can't improve the detail, it can either
-             * must return the values it already got as input, or issue an error,
+             * If the implementation can't improve the detail, it must either
+             * return the values it already got as input, or issue an error,
              * and we'll assume it couldn't get any better value. */
             MM_IFACE_MODEM_CDMA_GET_INTERFACE (ctx->self)->get_detailed_registration_state (
                 ctx->self,
@@ -849,23 +889,6 @@ registration_check_step (RunRegistrationChecksContext *ctx)
         mm_iface_modem_cdma_update_evdo_registration_state (ctx->self,
                                                             ctx->evdo_state);
 
-        /* Update access technologies.
-         * TODO: proper EV-DO revision reporting */
-        {
-            MMModemAccessTechnology act = MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN;
-
-            if (ctx->cdma1x_state == MM_MODEM_CDMA_REGISTRATION_STATE_HOME ||
-                ctx->cdma1x_state == MM_MODEM_CDMA_REGISTRATION_STATE_ROAMING ||
-                ctx->cdma1x_state == MM_MODEM_CDMA_REGISTRATION_STATE_REGISTERED)
-                act |= MM_IFACE_MODEM_CDMA_ALL_CDMA1X_ACCESS_TECHNOLOGIES_MASK;
-
-            if (ctx->evdo_state == MM_MODEM_CDMA_REGISTRATION_STATE_HOME ||
-                ctx->evdo_state == MM_MODEM_CDMA_REGISTRATION_STATE_ROAMING ||
-                ctx->evdo_state == MM_MODEM_CDMA_REGISTRATION_STATE_REGISTERED)
-                act |= MM_IFACE_MODEM_CDMA_ALL_EVDO_ACCESS_TECHNOLOGIES_MASK;
-            mm_iface_modem_cdma_update_access_technologies (MM_IFACE_MODEM_CDMA (ctx->self), act);
-        }
-
         g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
         run_registration_checks_context_complete_and_free (ctx);
         return;
@@ -1109,7 +1132,8 @@ periodic_registration_checks_ready (MMIfaceModemCdma *self,
 
     /* Remove the running tag */
     ctx = g_object_get_qdata (G_OBJECT (self), registration_check_context_quark);
-    ctx->running = FALSE;
+    if (ctx)
+        ctx->running = FALSE;
 }
 
 static gboolean
@@ -1173,6 +1197,55 @@ periodic_registration_check_enable (MMIfaceModemCdma *self)
 
 /*****************************************************************************/
 
+static GVariant *
+build_empty_dictionary (void)
+{
+    GVariantBuilder builder;
+
+    g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
+    return g_variant_builder_end (&builder);
+}
+
+void
+mm_iface_modem_cdma_update_activation_state (MMIfaceModemCdma *self,
+                                             MMModemCdmaActivationState activation_state,
+                                             const GError *activation_error)
+{
+    MmGdbusModemCdma *skeleton = NULL;
+    MMCdmaActivationError error = MM_CDMA_ACTIVATION_ERROR_NONE;
+
+    g_object_get (self,
+                  MM_IFACE_MODEM_CDMA_DBUS_SKELETON, &skeleton,
+                  NULL);
+    if (!skeleton)
+        return;
+
+    if (activation_error) {
+        mm_dbg ("Activation failed: %s", activation_error->message);
+        if (activation_error->domain == MM_CDMA_ACTIVATION_ERROR)
+            error = activation_error->code;
+        else {
+            mm_warn ("Error given is not an activation error");
+            error = MM_CDMA_ACTIVATION_ERROR_UNKNOWN;
+        }
+    }
+
+    /* Flush current change before signaling the state change,
+     * so that clients get the proper state already in the
+     * state-changed callback */
+    mm_gdbus_modem_cdma_set_activation_state (skeleton, activation_state);
+    g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (skeleton));
+    /* We don't know what changed, so just return an empty dictionary for now */
+    mm_gdbus_modem_cdma_emit_activation_state_changed (skeleton,
+                                                       activation_state,
+                                                       error,
+                                                       build_empty_dictionary ());
+
+    g_object_unref (skeleton);
+}
+
+/*****************************************************************************/
+
 typedef struct _DisablingContext DisablingContext;
 static void interface_disabling_step (DisablingContext *ctx);
 
@@ -1531,6 +1604,7 @@ typedef enum {
     INITIALIZATION_STEP_FIRST,
     INITIALIZATION_STEP_MEID,
     INITIALIZATION_STEP_ESN,
+    INITIALIZATION_STEP_ACTIVATION_STATE,
     INITIALIZATION_STEP_LAST
 } InitializationStep;
 
@@ -1594,6 +1668,27 @@ initialization_context_complete_and_free_if_cancelled (InitializationContext *ct
 STR_REPLY_READY_FN (meid, "MEID")
 STR_REPLY_READY_FN (esn, "ESN")
 
+static void
+load_activation_state_ready (MMIfaceModemCdma *self,
+                             GAsyncResult *res,
+                             InitializationContext *ctx)
+{
+    GError *error = NULL;
+    MMModemCdmaActivationState state;
+
+    state = MM_IFACE_MODEM_CDMA_GET_INTERFACE (self)->load_activation_state_finish (self, res, &error);
+    mm_gdbus_modem_cdma_set_activation_state (ctx->skeleton, state);
+
+    if (error) {
+        mm_warn ("couldn't load activation state: '%s'", error->message);
+        g_error_free (error);
+    }
+
+    /* Go on to next step */
+    ctx->step++;
+    interface_initialization_step (ctx);
+}
+
 static void
 interface_initialization_step (InitializationContext *ctx)
 {
@@ -1638,6 +1733,22 @@ interface_initialization_step (InitializationContext *ctx)
         /* Fall down to next step */
         ctx->step++;
 
+    case INITIALIZATION_STEP_ACTIVATION_STATE:
+        /* Initial activation state is meant to be loaded only once during the
+         * whole lifetime of the modem. Therefore, if we already have it loaded,
+         * don't try to load it again. */
+        if (mm_gdbus_modem_cdma_get_activation_state (ctx->skeleton) == MM_MODEM_CDMA_ACTIVATION_STATE_UNKNOWN &&
+            MM_IFACE_MODEM_CDMA_GET_INTERFACE (ctx->self)->load_activation_state &&
+            MM_IFACE_MODEM_CDMA_GET_INTERFACE (ctx->self)->load_activation_state_finish) {
+            MM_IFACE_MODEM_CDMA_GET_INTERFACE (ctx->self)->load_activation_state (
+                ctx->self,
+                (GAsyncReadyCallback)load_activation_state_ready,
+                ctx);
+            return;
+        }
+        /* Fall down to next step */
+        ctx->step++;
+
     case INITIALIZATION_STEP_LAST:
         /* We are done without errors! */
 
@@ -1692,6 +1803,7 @@ mm_iface_modem_cdma_initialize (MMIfaceModemCdma *self,
         mm_gdbus_modem_cdma_set_esn (skeleton, NULL);
         mm_gdbus_modem_cdma_set_sid (skeleton, MM_MODEM_CDMA_SID_UNKNOWN);
         mm_gdbus_modem_cdma_set_nid (skeleton, MM_MODEM_CDMA_NID_UNKNOWN);
+        mm_gdbus_modem_cdma_set_activation_state (skeleton, MM_MODEM_CDMA_ACTIVATION_STATE_UNKNOWN);
 
         /* Bind our Registration State properties */
         g_object_bind_property (self, MM_IFACE_MODEM_CDMA_CDMA1X_REGISTRATION_STATE,
