@@ -42,13 +42,13 @@ typedef struct _MMIfaceModem MMIfaceModem;
 struct _MMIfaceModem {
     GTypeInterface g_iface;
 
-    /* Loading of the ModemCapabilities property */
-    void (*load_modem_capabilities) (MMIfaceModem *self,
-                                     GAsyncReadyCallback callback,
-                                     gpointer user_data);
-    MMModemCapability (*load_modem_capabilities_finish) (MMIfaceModem *self,
-                                                         GAsyncResult *res,
-                                                         GError **error);
+    /* Loading of the SupportedCapabilities property */
+    void (*load_supported_capabilities) (MMIfaceModem *self,
+                                         GAsyncReadyCallback callback,
+                                         gpointer user_data);
+    GArray * (*load_supported_capabilities_finish) (MMIfaceModem *self,
+                                                    GAsyncResult *res,
+                                                    GError **error);
 
     /* Loading of the CurrentCapabilities property */
     void (*load_current_capabilities) (MMIfaceModem *self,
@@ -58,14 +58,6 @@ struct _MMIfaceModem {
                                                            GAsyncResult *res,
                                                            GError **error);
 
-    /* Asynchronous modem power-down operation run during initialization */
-    void (*modem_init_power_down) (MMIfaceModem *self,
-                                   GAsyncReadyCallback callback,
-                                   gpointer user_data);
-    gboolean (*modem_init_power_down_finish) (MMIfaceModem *self,
-                                              GAsyncResult *res,
-                                              GError **error);
-
     /* Loading of the Manufacturer property */
     void (*load_manufacturer) (MMIfaceModem *self,
                                GAsyncReadyCallback callback,
@@ -134,15 +126,15 @@ struct _MMIfaceModem {
     void (*load_supported_modes) (MMIfaceModem *self,
                                   GAsyncReadyCallback callback,
                                   gpointer user_data);
-    MMModemMode (*load_supported_modes_finish) (MMIfaceModem *self,
-                                                GAsyncResult *res,
-                                                GError **error);
+    GArray * (*load_supported_modes_finish) (MMIfaceModem *self,
+                                             GAsyncResult *res,
+                                             GError **error);
 
-    /* Loading of the AllowedModes and PreferredMode properties */
-    void (*load_allowed_modes) (MMIfaceModem *self,
+    /* Loading of the Modes property */
+    void (*load_current_modes) (MMIfaceModem *self,
                                 GAsyncReadyCallback callback,
                                 gpointer user_data);
-    gboolean (*load_allowed_modes_finish) (MMIfaceModem *self,
+    gboolean (*load_current_modes_finish) (MMIfaceModem *self,
                                            GAsyncResult *res,
                                            MMModemMode *allowed,
                                            MMModemMode *preferred,
@@ -164,6 +156,22 @@ struct _MMIfaceModem {
                                            GAsyncResult *res,
                                            GError **error);
 
+    /* Loading of the SupportedIpFamilies property */
+    void (* load_supported_ip_families) (MMIfaceModem *self,
+                                         GAsyncReadyCallback callback,
+                                         gpointer user_data);
+    MMBearerIpFamily (* load_supported_ip_families_finish) (MMIfaceModem *self,
+                                                           GAsyncResult *res,
+                                                           GError **error);
+
+    /* Loading of the PowerState property */
+    void (* load_power_state) (MMIfaceModem *self,
+                               GAsyncReadyCallback callback,
+                               gpointer user_data);
+    MMModemPowerState (*load_power_state_finish) (MMIfaceModem *self,
+                                                  GAsyncResult *res,
+                                                  GError **error);
+
     /* Loading of the SignalQuality property */
     void  (*load_signal_quality) (MMIfaceModem *self,
                                   GAsyncReadyCallback callback,
@@ -209,33 +217,34 @@ struct _MMIfaceModem {
                                      GAsyncResult *res,
                                      GError **error);
 
-    /* Asynchronous allowed band setting operation */
-    void (*set_bands) (MMIfaceModem *self,
-                       GArray *bands_array,
-                       GAsyncReadyCallback callback,
-                       gpointer user_data);
-    gboolean (*set_bands_finish) (MMIfaceModem *self,
-                                  GAsyncResult *res,
-                                  GError **error);
+    /* Asynchronous capabilities setting operation */
+    void (*set_current_capabilities) (MMIfaceModem *self,
+                                      MMModemCapability,
+                                      GAsyncReadyCallback callback,
+                                      gpointer user_data);
+    gboolean (*set_current_capabilities_finish) (MMIfaceModem *self,
+                                                 GAsyncResult *res,
+                                                 GError **error);
 
-    /* Asynchronous allowed mode setting operation */
-    void (*set_allowed_modes) (MMIfaceModem *self,
+    /* Asynchronous current band setting operation */
+    void (*set_current_bands) (MMIfaceModem *self,
+                               GArray *bands_array,
+                               GAsyncReadyCallback callback,
+                               gpointer user_data);
+    gboolean (*set_current_bands_finish) (MMIfaceModem *self,
+                                          GAsyncResult *res,
+                                          GError **error);
+
+    /* Asynchronous current mode setting operation */
+    void (*set_current_modes) (MMIfaceModem *self,
                                MMModemMode modes,
                                MMModemMode preferred,
                                GAsyncReadyCallback callback,
                                gpointer user_data);
-    gboolean (*set_allowed_modes_finish) (MMIfaceModem *self,
+    gboolean (*set_current_modes_finish) (MMIfaceModem *self,
                                           GAsyncResult *res,
                                           GError **error);
 
-    /* Asynchronous modem initialization operation */
-    void (*modem_init) (MMIfaceModem *self,
-                        GAsyncReadyCallback callback,
-                        gpointer user_data);
-    gboolean (*modem_init_finish) (MMIfaceModem *self,
-                                   GAsyncResult *res,
-                                   GError **error);
-
     /* Asynchronous method to wait for the SIM to be ready after having
      * unlocked it. */
     void (*modem_after_sim_unlock) (MMIfaceModem *self,
@@ -315,6 +324,9 @@ struct _MMIfaceModem {
 
 GType mm_iface_modem_get_type (void);
 
+/* Helpers to query access technologies */
+MMModemAccessTechnology mm_iface_modem_get_access_technologies (MMIfaceModem *self);
+
 /* Helpers to query capabilities */
 MMModemCapability mm_iface_modem_get_current_capabilities (MMIfaceModem *self);
 gboolean          mm_iface_modem_is_3gpp                  (MMIfaceModem *self);
@@ -325,13 +337,12 @@ gboolean          mm_iface_modem_is_cdma                  (MMIfaceModem *self);
 gboolean          mm_iface_modem_is_cdma_only             (MMIfaceModem *self);
 
 /* Helpers to query supported modes */
-MMModemMode mm_iface_modem_get_supported_modes (MMIfaceModem *self);
-gboolean    mm_iface_modem_is_2g               (MMIfaceModem *self);
-gboolean    mm_iface_modem_is_2g_only          (MMIfaceModem *self);
-gboolean    mm_iface_modem_is_3g               (MMIfaceModem *self);
-gboolean    mm_iface_modem_is_3g_only          (MMIfaceModem *self);
-gboolean    mm_iface_modem_is_4g               (MMIfaceModem *self);
-gboolean    mm_iface_modem_is_4g_only          (MMIfaceModem *self);
+gboolean mm_iface_modem_is_2g      (MMIfaceModem *self);
+gboolean mm_iface_modem_is_2g_only (MMIfaceModem *self);
+gboolean mm_iface_modem_is_3g      (MMIfaceModem *self);
+gboolean mm_iface_modem_is_3g_only (MMIfaceModem *self);
+gboolean mm_iface_modem_is_4g      (MMIfaceModem *self);
+gboolean mm_iface_modem_is_4g_only (MMIfaceModem *self);
 
 /* Initialize Modem interface (async) */
 void     mm_iface_modem_initialize        (MMIfaceModem *self,
@@ -359,26 +370,29 @@ gboolean mm_iface_modem_disable_finish (MMIfaceModem *self,
                                         GAsyncResult *res,
                                         GError **error);
 
-/* Shutdown Modem interface */
-void mm_iface_modem_shutdown (MMIfaceModem *self);
-
-/* Request unlock recheck.
- * It will not only return the lock status, but also set the property values
- * in the DBus interface. */
-void        mm_iface_modem_unlock_check        (MMIfaceModem *self,
+/* Allow setting power state */
+void     mm_iface_modem_set_power_state        (MMIfaceModem *self,
+                                                MMModemPowerState power_state,
                                                 GAsyncReadyCallback callback,
                                                 gpointer user_data);
-MMModemLock mm_iface_modem_unlock_check_finish (MMIfaceModem *self,
+gboolean mm_iface_modem_set_power_state_finish (MMIfaceModem *self,
                                                 GAsyncResult *res,
                                                 GError **error);
 
-/* Check unlock retries */
-void mm_iface_modem_update_unlock_retries            (MMIfaceModem *self,
-                                                      GAsyncReadyCallback callback,
-                                                      gpointer user_data);
-gboolean mm_iface_modem_update_unlock_retries_finish (MMIfaceModem *self,
-                                                      GAsyncResult *res,
-                                                      GError **error);
+/* Shutdown Modem interface */
+void mm_iface_modem_shutdown (MMIfaceModem *self);
+
+/* Request lock info update.
+ * It will not only return the lock status, but also set the property values
+ * in the DBus interface. If 'known_lock' is given, that lock status will be
+ * assumed. */
+void        mm_iface_modem_update_lock_info        (MMIfaceModem *self,
+                                                    MMModemLock known_lock,
+                                                    GAsyncReadyCallback callback,
+                                                    gpointer user_data);
+MMModemLock mm_iface_modem_update_lock_info_finish (MMIfaceModem *self,
+                                                    GAsyncResult *res,
+                                                    GError **error);
 
 /* Request signal quality check update.
  * It will not only return the signal quality status, but also set the property
@@ -399,6 +413,8 @@ void mm_iface_modem_update_subsystem_state (MMIfaceModem *self,
 void mm_iface_modem_update_state (MMIfaceModem *self,
                                   MMModemState new_state,
                                   MMModemStateChangeReason reason);
+void mm_iface_modem_update_failed_state (MMIfaceModem *self,
+                                         MMModemStateFailedReason failed_reason);
 
 /* Allow reporting new access tech */
 void mm_iface_modem_update_access_technologies (MMIfaceModem *self,
@@ -410,23 +426,23 @@ void mm_iface_modem_update_signal_quality (MMIfaceModem *self,
                                            guint signal_quality);
 
 /* Allow setting allowed modes */
-void     mm_iface_modem_set_allowed_modes        (MMIfaceModem *self,
+void     mm_iface_modem_set_current_modes        (MMIfaceModem *self,
                                                   MMModemMode allowed,
                                                   MMModemMode preferred,
                                                   GAsyncReadyCallback callback,
                                                   gpointer user_data);
-gboolean mm_iface_modem_set_allowed_modes_finish (MMIfaceModem *self,
+gboolean mm_iface_modem_set_current_modes_finish (MMIfaceModem *self,
                                                   GAsyncResult *res,
                                                   GError **error);
 
 /* Allow setting bands */
-void     mm_iface_modem_set_bands        (MMIfaceModem *self,
-                                          GArray *bands_array,
-                                          GAsyncReadyCallback callback,
-                                          gpointer user_data);
-gboolean mm_iface_modem_set_bands_finish (MMIfaceModem *self,
-                                          GAsyncResult *res,
-                                          GError **error);
+void     mm_iface_modem_set_current_bands        (MMIfaceModem *self,
+                                                  GArray *bands_array,
+                                                  GAsyncReadyCallback callback,
+                                                  gpointer user_data);
+gboolean mm_iface_modem_set_current_bands_finish (MMIfaceModem *self,
+                                                  GAsyncResult *res,
+                                                  GError **error);
 
 /* Allow creating bearers */
 void     mm_iface_modem_create_bearer         (MMIfaceModem *self,
