@@ -26,6 +26,8 @@
 
 MMModemCapability mm_modem_capability_from_qmi_radio_interface (QmiDmsRadioInterface network);
 
+MMModemMode mm_modem_mode_from_qmi_radio_interface (QmiDmsRadioInterface network);
+
 MMModemLock mm_modem_lock_from_qmi_uim_pin_status (QmiDmsUimPinStatus status,
                                                        gboolean pin1);
 
@@ -53,8 +55,10 @@ QmiNasRatModePreference mm_modem_mode_to_qmi_rat_mode_preference (MMModemMode mo
                                                                   gboolean is_3gpp);
 
 MMModemCapability mm_modem_capability_from_qmi_rat_mode_preference (QmiNasRatModePreference qmi);
+QmiNasRatModePreference mm_modem_capability_to_qmi_rat_mode_preference (MMModemCapability caps);
 
 MMModemCapability mm_modem_capability_from_qmi_radio_technology_preference (QmiNasRadioTechnologyPreference qmi);
+QmiNasRadioTechnologyPreference mm_modem_capability_to_qmi_radio_technology_preference (MMModemCapability caps);
 
 MMModemCapability mm_modem_capability_from_qmi_band_preference (QmiNasBandPreference qmi);
 
@@ -75,6 +79,8 @@ MMModem3gppRegistrationState mm_modem_3gpp_registration_state_from_qmi_registrat
 
 MMModemCdmaRegistrationState mm_modem_cdma_registration_state_from_qmi_registration_state (QmiNasRegistrationState registration_state);
 
+MMModemCdmaActivationState mm_modem_cdma_activation_state_from_qmi_activation_state (QmiDmsActivationState state);
+
 /*****************************************************************************/
 /* QMI/WMS to MM translations */
 
@@ -88,4 +94,18 @@ MMSmsState mm_sms_state_from_qmi_message_tag (QmiWmsMessageTagType tag);
 
 QmiWdsAuthentication mm_bearer_allowed_auth_to_qmi_authentication (MMBearerAllowedAuth auth);
 
+/*****************************************************************************/
+/* Utility to gather current capabilities from various sources */
+
+typedef struct {
+    /* NAS System Selection Preference */
+    QmiNasRatModePreference nas_ssp_mode_preference_mask;
+    /* NAS Technology Preference */
+    QmiNasRadioTechnologyPreference nas_tp_mask;
+    /* DMS Capabilities */
+    MMModemCapability dms_capabilities;
+} MMQmiCapabilitiesContext;
+
+MMModemCapability mm_modem_capability_from_qmi_capabilities_context (MMQmiCapabilitiesContext *ctx);
+
 #endif  /* MM_MODEM_HELPERS_QMI_H */
