@@ -76,10 +76,13 @@ struct _MMPluginPrivate {
     gboolean at;
     gboolean single_at;
     gboolean qcdm;
+    gboolean qmi;
+    gboolean mbim;
     gboolean icera_probe;
     MMPortProbeAtCommand *custom_at_probe;
     guint64 send_delay;
     gboolean remove_echo;
+    gboolean send_lf;
 
     /* Probing setup and/or post-probing filter.
      * Plugins may use this method to decide whether they support a given
@@ -103,6 +106,8 @@ enum {
     PROP_ALLOWED_AT,
     PROP_ALLOWED_SINGLE_AT,
     PROP_ALLOWED_QCDM,
+    PROP_ALLOWED_QMI,
+    PROP_ALLOWED_MBIM,
     PROP_ICERA_PROBE,
     PROP_ALLOWED_ICERA,
     PROP_FORBIDDEN_ICERA,
@@ -110,6 +115,7 @@ enum {
     PROP_CUSTOM_INIT,
     PROP_SEND_DELAY,
     PROP_REMOVE_ECHO,
+    PROP_SEND_LF,
     LAST_PROP
 };
 
@@ -155,6 +161,33 @@ is_virtual_port (const gchar *device_name)
     return FALSE;
 }
 
+/* Returns TRUE if the support check request was filtered out */
+static gboolean
+apply_subsystem_filter (MMPlugin *self,
+                        GUdevDevice *port)
+{
+    if (self->priv->subsystems) {
+        const gchar *subsys;
+        guint i;
+
+        subsys = g_udev_device_get_subsystem (port);
+        for (i = 0; self->priv->subsystems[i]; i++) {
+            if (g_str_equal (subsys, self->priv->subsystems[i]))
+                break;
+            /* New kernels may report as 'usbmisc' the subsystem */
+            else if (g_str_equal (self->priv->subsystems[i], "usb") &&
+                     g_str_equal (subsys, "usbmisc"))
+                break;
+        }
+
+        /* If we didn't match any subsystem: unsupported */
+        if (!self->priv->subsystems[i])
+            return TRUE;
+    }
+
+    return FALSE;
+}
+
 /* Returns TRUE if the support check request was filtered out */
 static gboolean
 apply_pre_probing_filters (MMPlugin *self,
@@ -174,26 +207,11 @@ apply_pre_probing_filters (MMPlugin *self,
 
     /* The plugin may specify that only some subsystems are supported. If that
      * is the case, filter by subsystem */
-    if (self->priv->subsystems) {
-        const gchar *subsys;
-
-        subsys = g_udev_device_get_subsystem (port);
-        for (i = 0; self->priv->subsystems[i]; i++) {
-            if (g_str_equal (subsys, self->priv->subsystems[i]))
-                break;
-            /* New kernels may report as 'usbmisc' the subsystem */
-            else if (g_str_equal (self->priv->subsystems[i], "usb") &&
-                     g_str_equal (subsys, "usbmisc"))
-                break;
-        }
-
-        /* If we didn't match any subsystem: unsupported */
-        if (!self->priv->subsystems[i]) {
-            mm_dbg ("(%s) [%s] filtered by subsystem",
-                    self->priv->name,
-                    g_udev_device_get_name (port));
-            return TRUE;
-        }
+    if (apply_subsystem_filter (self, port)) {
+        mm_dbg ("(%s) [%s] filtered by subsystem",
+                self->priv->name,
+                g_udev_device_get_name (port));
+        return TRUE;
     }
 
     /* The plugin may specify that only some drivers are supported, or that some
@@ -534,6 +552,7 @@ port_probe_run_ready (MMPortProbe *probe,
             g_simple_async_result_set_op_res_gpointer (ctx->result,
                                                        GUINT_TO_POINTER (MM_PLUGIN_SUPPORTS_PORT_UNSUPPORTED),
                                                        NULL);
+            g_error_free (error);
         }
         /* Probing failed but the plugin tells us to retry; so we'll defer the
          * probing a bit */
@@ -543,6 +562,7 @@ port_probe_run_ready (MMPortProbe *probe,
             g_simple_async_result_set_op_res_gpointer (ctx->result,
                                                        GUINT_TO_POINTER (MM_PLUGIN_SUPPORTS_PORT_DEFER),
                                                        NULL);
+            g_error_free (error);
         }
         /* For remaining errors, just propagate them */
         else {
@@ -608,6 +628,21 @@ mm_plugin_supports_port_finish (MMPlugin *self,
     return (MMPluginSupportsResult) GPOINTER_TO_UINT (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (result)));
 }
 
+static gboolean
+find_driver_in_device (MMDevice *device,
+                       const gchar *driver)
+{
+    const gchar **device_drivers;
+    guint i;
+
+    device_drivers = mm_device_get_drivers (device);
+    for (i = 0; device_drivers[i]; i++) {
+        if (g_str_equal (driver, device_drivers[i]))
+            return TRUE;
+    }
+    return FALSE;
+}
+
 void
 mm_plugin_supports_port (MMPlugin *self,
                          MMDevice *device,
@@ -615,7 +650,7 @@ mm_plugin_supports_port (MMPlugin *self,
                          GAsyncReadyCallback callback,
                          gpointer user_data)
 {
-    MMPortProbe *probe;
+    MMPortProbe *probe = NULL;
     GSimpleAsyncResult *async_result;
     PortProbeRunContext *ctx;
     gboolean need_vendor_probing;
@@ -689,10 +724,22 @@ mm_plugin_supports_port (MMPlugin *self,
             probe_run_flags |= (MM_PORT_PROBE_AT | MM_PORT_PROBE_AT_ICERA);
     } else {
         /* cdc-wdm ports... */
-        probe_run_flags = MM_PORT_PROBE_QMI;
+        probe_run_flags = MM_PORT_PROBE_NONE;
+        if (self->priv->qmi && find_driver_in_device (device, "qmi_wwan"))
+            probe_run_flags |= MM_PORT_PROBE_QMI;
+        if (self->priv->mbim && find_driver_in_device (device, "cdc_mbim"))
+            probe_run_flags |= MM_PORT_PROBE_MBIM;
     }
 
-    g_assert (probe_run_flags != MM_PORT_PROBE_NONE);
+    /* If no explicit probing was required, just request to grab it without probing anything.
+     * This may happen, e.g. with cdc-wdm ports which do not need QMI/MBIM probing. */
+    if (probe_run_flags == MM_PORT_PROBE_NONE) {
+        g_simple_async_result_set_op_res_gpointer (async_result,
+                                                   GUINT_TO_POINTER (MM_PLUGIN_SUPPORTS_PORT_DEFER_UNTIL_SUGGESTED),
+                                                   NULL);
+        g_simple_async_result_complete_in_idle (async_result);
+        goto out;
+    }
 
     /* If a modem is already available and the plugin says that only one AT port is
      * expected, check if we alredy got the single AT port. And if so, we know this
@@ -729,6 +776,7 @@ mm_plugin_supports_port (MMPlugin *self,
                        ctx->flags,
                        self->priv->send_delay,
                        self->priv->remove_echo,
+                       self->priv->send_lf,
                        self->priv->custom_at_probe,
                        self->priv->custom_init,
                        (GAsyncReadyCallback)port_probe_run_ready,
@@ -736,6 +784,8 @@ mm_plugin_supports_port (MMPlugin *self,
 
 out:
     g_object_unref (async_result);
+    if (probe)
+        g_object_unref (probe);
 }
 
 /*****************************************************************************/
@@ -790,6 +840,8 @@ mm_plugin_create_modem (MMPlugin  *self,
                                                       port_probes,
                                                       error);
     if (modem) {
+        mm_base_modem_set_hotplugged (modem, mm_device_get_hotplugged (device));
+
         /* Grab each port */
         for (l = port_probes; l; l = g_list_next (l)) {
             GError *inner_error = NULL;
@@ -799,7 +851,37 @@ mm_plugin_create_modem (MMPlugin  *self,
             /* If grabbing a port fails, just warn. We'll decide if the modem is
              * valid or not when all ports get organized */
 
-            if (MM_PLUGIN_GET_CLASS (self)->grab_port)
+            /* We apply again the subsystem filter, as the port may have been
+             * probed and accepted by the generic plugin, which is overwritten
+             * by the specific one when needed. */
+            if (apply_subsystem_filter (self, mm_port_probe_peek_port (probe))) {
+                grabbed = FALSE;
+                inner_error = g_error_new (MM_CORE_ERROR,
+                                           MM_CORE_ERROR_UNSUPPORTED,
+                                           "unsupported subsystem: '%s'",
+                                           mm_port_probe_get_port_subsys (probe));
+            }
+#if !defined WITH_QMI
+            else if (mm_port_probe_get_port_type (probe) == MM_PORT_TYPE_NET &&
+                     g_str_equal (mm_device_utils_get_port_driver (mm_port_probe_peek_port (probe)),
+                                  "qmi_wwan")) {
+                grabbed = FALSE;
+                inner_error = g_error_new (MM_CORE_ERROR,
+                                           MM_CORE_ERROR_UNSUPPORTED,
+                                           "ignoring QMI net port");
+            }
+#endif
+#if !defined WITH_MBIM
+            else if (mm_port_probe_get_port_type (probe) == MM_PORT_TYPE_NET &&
+                     g_str_equal (mm_device_utils_get_port_driver (mm_port_probe_peek_port (probe)),
+                                  "cdc_mbim")) {
+                grabbed = FALSE;
+                inner_error = g_error_new (MM_CORE_ERROR,
+                                           MM_CORE_ERROR_UNSUPPORTED,
+                                           "ignoring MBIM net port");
+            }
+#endif
+            else if (MM_PLUGIN_GET_CLASS (self)->grab_port)
                 grabbed = MM_PLUGIN_GET_CLASS (self)->grab_port (MM_PLUGIN (self),
                                                                  modem,
                                                                  probe,
@@ -840,6 +922,7 @@ mm_plugin_init (MMPlugin *self)
     /* Defaults */
     self->priv->send_delay = 100000;
     self->priv->remove_echo = TRUE;
+    self->priv->send_lf = FALSE;
 }
 
 static void
@@ -907,6 +990,14 @@ set_property (GObject *object,
         /* Construct only */
         self->priv->qcdm = g_value_get_boolean (value);
         break;
+    case PROP_ALLOWED_QMI:
+        /* Construct only */
+        self->priv->qmi = g_value_get_boolean (value);
+        break;
+    case PROP_ALLOWED_MBIM:
+        /* Construct only */
+        self->priv->mbim = g_value_get_boolean (value);
+        break;
     case PROP_ICERA_PROBE:
         /* Construct only */
         self->priv->icera_probe = g_value_get_boolean (value);
@@ -935,6 +1026,10 @@ set_property (GObject *object,
         /* Construct only */
         self->priv->remove_echo = g_value_get_boolean (value);
         break;
+    case PROP_SEND_LF:
+        /* Construct only */
+        self->priv->send_lf = g_value_get_boolean (value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -989,6 +1084,12 @@ get_property (GObject *object,
     case PROP_ALLOWED_QCDM:
         g_value_set_boolean (value, self->priv->qcdm);
         break;
+    case PROP_ALLOWED_QMI:
+        g_value_set_boolean (value, self->priv->qmi);
+        break;
+    case PROP_ALLOWED_MBIM:
+        g_value_set_boolean (value, self->priv->mbim);
+        break;
     case PROP_ALLOWED_UDEV_TAGS:
         g_value_set_boxed (value, self->priv->udev_tags);
         break;
@@ -1013,6 +1114,9 @@ get_property (GObject *object,
     case PROP_REMOVE_ECHO:
         g_value_set_boolean (value, self->priv->remove_echo);
         break;
+    case PROP_SEND_LF:
+        g_value_set_boolean (value, self->priv->send_lf);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -1026,6 +1130,23 @@ finalize (GObject *object)
 
     g_free (self->priv->name);
 
+#define _g_boxed_free0(t,p) if (p) g_boxed_free (t, p)
+
+    _g_boxed_free0 (G_TYPE_STRV, self->priv->subsystems);
+    _g_boxed_free0 (G_TYPE_STRV, self->priv->drivers);
+    _g_boxed_free0 (G_TYPE_STRV, self->priv->forbidden_drivers);
+    _g_boxed_free0 (MM_TYPE_UINT16_ARRAY, self->priv->vendor_ids);
+    _g_boxed_free0 (MM_TYPE_UINT16_PAIR_ARRAY, self->priv->product_ids);
+    _g_boxed_free0 (MM_TYPE_UINT16_PAIR_ARRAY, self->priv->forbidden_product_ids);
+    _g_boxed_free0 (G_TYPE_STRV, self->priv->udev_tags);
+    _g_boxed_free0 (G_TYPE_STRV, self->priv->vendor_strings);
+    _g_boxed_free0 (MM_TYPE_STR_PAIR_ARRAY, self->priv->product_strings);
+    _g_boxed_free0 (MM_TYPE_STR_PAIR_ARRAY, self->priv->forbidden_product_strings);
+    _g_boxed_free0 (MM_TYPE_POINTER_ARRAY, self->priv->custom_at_probe);
+    _g_boxed_free0 (MM_TYPE_ASYNC_METHOD, self->priv->custom_init);
+
+#undef _g_boxed_free0
+
     G_OBJECT_CLASS (mm_plugin_parent_class)->finalize (object);
 }
 
@@ -1163,6 +1284,22 @@ mm_plugin_class_init (MMPluginClass *klass)
                                FALSE,
                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+    g_object_class_install_property
+        (object_class, PROP_ALLOWED_QMI,
+         g_param_spec_boolean (MM_PLUGIN_ALLOWED_QMI,
+                               "Allowed QMI",
+                               "Whether QMI ports are allowed in this plugin",
+                               FALSE,
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+    g_object_class_install_property
+        (object_class, PROP_ALLOWED_MBIM,
+         g_param_spec_boolean (MM_PLUGIN_ALLOWED_MBIM,
+                               "Allowed MBIM",
+                               "Whether MBIM ports are allowed in this plugin",
+                               FALSE,
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
     g_object_class_install_property
         (object_class, PROP_ICERA_PROBE,
          g_param_spec_boolean (MM_PLUGIN_ICERA_PROBE,
@@ -1222,4 +1359,12 @@ mm_plugin_class_init (MMPluginClass *klass)
                                "Remove echo out of the AT responses",
                                TRUE,
                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+    g_object_class_install_property
+        (object_class, PROP_SEND_LF,
+         g_param_spec_boolean (MM_PLUGIN_SEND_LF,
+                               "Send LF",
+                               "Send line-feed at the end of each AT command sent",
+                               FALSE,
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
